# $Header: /home/cvsroot/zSQLgate/DBIServer/Spec.pm,v 1.19 2005/04/08 22:59:10 mike Exp $

package Net::Z3950::DBIServer::Spec;
use strict;


=head1 NAME

Net::Z3950::DBIServer::Spec - The zSQLgate Configuration File Format


=head1 SYNOPSIS

C<zSQLgate> configuration files are free-form and treat all whitespace
equivalently, with the sole exception of comments.  Related directives
are grouped together with braces, with sections of detailed
information nested within more general sections.

=cut

### say more: e.g. one or more DBs each with search & data sections.

=head1 DESCRIPTION

Comments are introduced by a hash character (C<#>), and run to the end
of the line.  They are ignored.  Comments are the only part of the
configuration file syntax that treats lines specially: otherwise the
syntax is totally free-form.

The contents of the file must follow this BNF grammar, where C<config>
is the start production and C<@> represents the empty production.

	config		= chunk*

	chunk		= DATASOURCE '=' STRING
			| USERNAME '=' STRING
			| PASSWORD '=' STRING
			| perDb

	perDB		= DATABASE DBNAME '{' section* '}'

	section		= TABLE '=' TABLENAME
			| AUXILIARY TABLENAME WITH CONDITION
			| RESTRICTION '=' STRING
			| searchSpec
			| dataSpec

	searchSpec	= SEARCH attrsetSpec
			| SEARCH '{' attrsetSpec* '}'

	attrsetSpec	= ATTRSET oidSpec '{' accessSpec* '}'

	oidSpec		= oid
			| SYMBOL

	oid		= NUMBER
			| NUMBER '.' oid

	accessSpec	= ACCESS NUMBER FULLTEXT? '=' COLUMNNAME

	dataSpec	= DATA formatSpec
			| DATA '{' formatSpec* '}'

	formatSpec	= CACHESIZE NUMBER
			| FORMAT GRS1 grsFormatSpec
			| FORMAT XML xmlFormatSpec
			| FORMAT MARC xmlFormatSpec

	grsFormatSpec	= '{' grs1ElementSpec* '}'

	grs1ElementSpec	= COLUMN COLUMNNAME '=' tag+

	tag		= '(' NUMBER ',' tag-val ')'

	tag-val		= NUMBER
			| STRING

	xmlFormatSpec	= '{' xmlElementSpec* '}'

	xmlElementSpec	= RECORD '=' STRING
			| ATTRS '=' STRING
			| TRANSFORM '=' FILENAME
			| FIELD TAGNAME '=' COLUMNNAME
			| xmlColumnSpec # DEPRECATED

	xmlColumnSpec	= COLUMN COLUMNNAME '=' TAGNAME

	DBNAME		= STRING
	TABLENAME	= STRING
	CONDITION	= STRING
	COLUMNNAME	= STRING
	FILENAME	= STRING
	TAGNAME		= STRING

I make every effort to keep this BNF file up to date with respect to
the actual parsing code in C<Net::Z3950::DBIServer::Config> but I'm
not going to write a lot of prose describing what the various sections
mean yet, because I think this format is likely to change a fair bit
before it stabilises enough for that to be worthwhile.  See the next
section for examples of possible changes.


=head1 FUTURE DIRECTIONS

I anticipate extending the configuration file syntax in the following
ways:

=head2 Enhancements to Searching

=over 4

=item *

Allow Functional and Semantic Qualifiers (as specified in the
Attribute Architecture) to work along with the Use Attributes which
are currently specified to select the access point.  I'm not at all
sure yet how this might look in the config file.

Z<>

=item *

Allow specification of semantics for attributes other than access
points.  (Relation and truncation are handled by hardwired code that
knows about the relevant attributes in the BIB-1 and Utility sets.)

Z<>

=back

=head2 Enhancements to Retrieval

=over 4

=item *

Allow GRS-1 data specifications to indicate which schema is in use.
This generalises to allowing records to include arbitrary constant
data: in this case, a schema OID associated with the tag-path (1,1) -
that's tagSet-M's C<schemaIdentifier>.  If you don't know what that
means, it's probably because you prefer XML to GRS-1 :-)

Z<>

=item *

Allow XML data specifications to indicate which DTD (or Schema?) is in
use.  Again, I think this generalises to the inclusion of arbitrary
constant data - maybe just at the start of the record, but maybe we can
generalise the generalisation.

Z<>

=item *

Allow elements to be marked for inclusion in the brief record
(element-set name C<b>).  More generally, provide the wherewithal for
the configuration file to specify the contents of arbitrary element
sets.

Z<>

=back

=head2 Miscellaneous Enhancements

=over 4

=item *

Allow the configuration file to override the back-end data-source (DBI
database, or DSN) on a per-logical-database basis.

Z<>

=item *

Allow the configuration file to set other DBI parameters such as
C<auto_commit>.

Z<>

=item *

Allow the configuration file to specify which port or ports the server
should listen on.  Maybe also other YAZ server options.

Z<>

=item *

Allow specification of per-database authentication requirements.

Z<>

=item *

Allow the inclusion of sub-files.  This will be useful to allow, for
example, element sets to be specified in their own files.

Z<>

=item *

Provide the ability to make multiple back-end databases look like a
single large data repository (``union catalogue'').  This is a much
bigger deal the most of the other enhancements mentioned here, and
will probably only happen if there's a real need for it in a specific
project.

Z<>

=back

And no doubt many others that I'll think of later.


=head1 AUTHOR

Mike Taylor E<lt>mike@miketaylor.org.ukE<gt>

First version Sunday 24th February 2002.

=cut

1;
